import Color exposing (..)
import Graphics.Collage exposing (collage, filled, circle, rect, move)
import Signal exposing (..)
import Window
import Time exposing (..)
import Graphics.Element exposing (show)
import Mouse

(widthOfWall, heightOfWall) = (10, 200)
(paddleWidth, paddleHeight) = (50, 10)
framesPerSecond = 50
(deltaX, deltaY) = (7, -3)
ballRadius = 10

type alias Ball =
 { x : Float
 , y : Float
 , dx : Float
 , dy : Float
 }

type alias Paddle =
 {
   x: Float
 , y: Float
 }

drawWall wallWidth wallHeight shiftX shiftY =
  rect wallWidth wallHeight
    |> filled black
    |> move (shiftX, shiftY)

drawLeftWall width height =
  drawWall widthOfWall heightOfWall ((toFloat -width / 2) + widthOfWall) 0

drawRightWall width height =
  drawWall widthOfWall heightOfWall ((toFloat width / 2) - widthOfWall - widthOfWall) 0

drawFloor width height =
  drawWall (toFloat width - widthOfWall) widthOfWall -5 -100

drawBall ball =
  circle ballRadius
    |> filled red
    |> move (ball.x, ball.y)

drawPaddle paddle =
  rect paddleWidth paddleHeight
    |> filled blue
    |> move (paddle.x, paddle.y)

drawGame (time, (width, height), ball, paddle) =
 collage width height 
  [
  (drawLeftWall width height)
  , (drawFloor width height)
  , (drawRightWall width height)
  , (drawBall ball)
  , (drawPaddle paddle)
  ]

paddleLocation (width, height) mouse =
  let
    halfWidth = (toFloat width) / 2
  in
    ((width, height), Paddle (toFloat (fst mouse) - halfWidth) 100)

updatePaddleLocation =
  map2 paddleLocation Window.dimensions Mouse.position

startingBall =
  Ball 0 0 deltaX deltaY

combine time (dimensions, paddle) =
  (time, dimensions, paddle)

adjustForCollisions ((width, height), ball, paddle) =
  let
    halfWidth = toFloat width / 2
    halfHeight = toFloat heightOfWall / 2
  in
    if ball.x < -(halfWidth - 30) || ball.x > halfWidth - 30 then
    { ball |
        dx = -(ball.dx)
    }     
    else if ball.y < -(halfHeight - 20) then
    { ball |
        dy = -(ball.dy)
    }     
    else if ball.y + ballRadius <= halfHeight && ball.y + 2 * ballRadius > halfHeight && ball.x >= (paddle.x - paddleWidth / 2) && ball.x <= (paddle.x + paddleWidth / 2) then
    { ball |
        y = halfHeight - 2 * ballRadius,
        dy = -(ball.dy)
    }     
    else
      ball

offsetBall time (width, height) paddle ball =
  (time
  , (width, height)
  , {ball |
      x = ball.x + ball.dx,
      y = ball.y + ball.dy
    }
  , paddle)

updateBall (time, dimensions, paddle) (oldTime, oldDimensions, ball, oldPaddle) =
  if time > oldTime then
    adjustForCollisions (dimensions, ball, paddle)
      |> offsetBall time dimensions paddle
  else
    (time, oldDimensions, ball, oldPaddle)

pulse =
  foldp (\time count -> count + 1) 0 (fps framesPerSecond)
  
moveBall =
  foldp updateBall (0, (0, 0), startingBall, (Paddle 0 100)) (map2 combine pulse updatePaddleLocation)

main =
  map drawGame moveBall